using System;
using System.IO;
using System.Collections.Generic;
using System.Net;
using System.Security.Cryptography;
using System.Text;

public class BspAPI
{
    /* Basic request URL */
    private static readonly string URL = "csec.api.qcloud.com/v2/index.php";
    private static readonly Random random = new Random();

    /* Signature algorithm using HMAC-SHA1 */
    public static string HmacSHA1(string key, string text)
    {
        using (HMACSHA1 hmac = new HMACSHA1(Encoding.Default.GetBytes(key)))
            return Convert.ToBase64String(hmac.ComputeHash(Encoding.Default.GetBytes(text)));
    }

    /* Assemble query string */
    private static string MakeQueryString(SortedDictionary<string, string> args, bool isURLEncoded)
    {
        List<string> query = new List<string>();

        foreach (KeyValuePair<string, string> pair in args)
            query.Add(pair.Key + "=" + (isURLEncoded ? WebUtility.UrlEncode(pair.Value) : pair.Value));

        return string.Join("&", query);
    }

    /* Generates an available URL */
    public static string MakeURL(
        string method,
        string action,
        string region,
        string secretId,
        string secretKey,
        Dictionary<string, string> args)
    {
        SortedDictionary<string, string> dict = new SortedDictionary<string, string>(StringComparer.Ordinal);

        /* Add all parameters */
        foreach (KeyValuePair<string, string> pair in args)
            dict[pair.Key] = pair.Value;

        /* Add common parameters, then calculate signature */
        dict["Nonce"] = random.Next(0x7fffffff).ToString();
        dict["Action"] = action;
        dict["Region"] = region;
        dict["SecretId"] = secretId;
        dict["Timestamp"] = ((long)DateTime.UtcNow.Subtract(new DateTime(1970, 1, 1)).TotalSeconds).ToString();
        dict["Signature"] = HmacSHA1(secretKey, string.Format("{0}{1}?{2}", method, URL, MakeQueryString(dict, false)));

        /* Assemble final request URL */
        return string.Format("https://{0}?{1}", URL, MakeQueryString(dict, true));
    }

    /* Message-Struct constructor of API `UgcAntiSpam` */
    public static class Message
    {
        /* Basic TLV structure */
        public abstract class TLV
        {
            /* The "T" and "V" part of "TLV" */
            private readonly int    type;
            private readonly String value;

            public TLV(int type) : this(type, "") {}
            public TLV(int type, string value)
            {
                this.type  = type;
                this.value = value;
            }

            /* Assemble a TLV element */
            public byte[] assemble()
            {
                byte[] bytes = Encoding.Default.GetBytes(value);
                byte[] result = new byte[bytes.Length + 8];

                /* Type */
                result[0] = (byte)((type & 0xff000000) >> 24);
                result[1] = (byte)((type & 0x00ff0000) >> 16);
                result[2] = (byte)((type & 0x0000ff00) >>  8);
                result[3] = (byte)((type & 0x000000ff) >>  0);

                /* Length */
                result[4] = (byte)((bytes.Length & 0xff000000) >> 24);
                result[5] = (byte)((bytes.Length & 0x00ff0000) >> 16);
                result[6] = (byte)((bytes.Length & 0x0000ff00) >>  8);
                result[7] = (byte)((bytes.Length & 0x000000ff) >>  0);

                /* Value */
                Array.Copy(bytes, 0, result, 8, bytes.Length);
                return result;
            }
        }

        /* Message items of the Message-Struct */
        public class Content : TLV
        {
            public Content()             : base(1) {}
            public Content(String value) : base(1, value) {}
        }

        public class ImageURL : TLV
        {
            public ImageURL()             : base(2) {}
            public ImageURL(String value) : base(2, value) {}
        }

        public class VideoURL : TLV
        {
            public VideoURL()             : base(3) {}
            public VideoURL(String value) : base(3, value) {}
        }

        public class AudioURL : TLV
        {
            public AudioURL()             : base(5) {}
            public AudioURL(String value) : base(4, value) {}
        }

        public class WebsiteURL : TLV
        {
            public WebsiteURL()             : base(5) {}
            public WebsiteURL(String value) : base(5, value) {}
        }

        public class Emotion : TLV
        {
            public Emotion()             : base(6) {}
            public Emotion(String value) : base(6, value) {}
        }

        public class Title : TLV
        {
            public Title()             : base(7) {}
            public Title(String value) : base(7, value) {}
        }

        public class Location : TLV { public Location() : base(8) {} }
        public class Custom   : TLV { public Custom()   : base(9) {} }
        public class File     : TLV { public File()     : base(10) {} }
        public class Other    : TLV { public Other()    : base(1000) {} }

        /* Factory method of Message-Struct */
        public static string build(params TLV[] items)
        {
            int length = 0;
            List<byte[]> parts = new List<byte[]>();

            /* Assemble each part */
            foreach (TLV item in items)
            {
                byte[] part = item.assemble();

                parts.Add(part);
                length += part.Length;
            }

            int current = 0;
            byte[] result = new byte[length];

            /* Copy to result */
            foreach (byte[] part in parts)
            {
                Array.Copy(part, 0, result, current, part.Length);
                current += part.Length;
            }

            /* Encode using Base64 */
            return Convert.ToBase64String(result);
        }
    }
}

/* Demo section */
public class MainDemo
{
    /**
     * GET 请求
     */
    private static string SendGet(string url)
    {
        HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
        request.Method = "GET";
        HttpWebResponse response = request.GetResponse() as HttpWebResponse;
        Stream receiveStream = response.GetResponseStream();
        Encoding encode = Encoding.GetEncoding("utf-8");
        StreamReader readStream = new StreamReader(receiveStream, encode);
        string result = readStream.ReadToEnd();
        response.Close();
        readStream.Close();
        return result;
    }
    public static void Main(string[] argv)
    {
        /* 密钥,请进行替换,密钥申请地址 https://console.qcloud.com/capi  */
        const string SECRET_ID = "";
        const string SECRET_KEY = "";

        Dictionary<string, string> args = new Dictionary<string, string>();

        /* 账号信息 */
        args["accountType"]          = "5";
        args["uid"]                  = "D692D87319F2098C3877C3904B304706";
        args["associateAccount"]     = "373909726";
        args["nickName"]             = "helloword";
        args["phoneNumber"]          = "086+15166666666";
        args["emailAddress"]         = "hellword@qq.com";
        args["registerTime"]         = "1440416972";
        args["registerIp"]           = "121.14.96.121";
        args["passwordHash"]         = "f158abb2a762f7919846ee9bf8445c7f22a244c5";

        /* 行为信息 */
        args["registerTime"]         = "1440416972";
        args["registerIp"]           = "121.14.96.121";
        args["registerSource"]       = "2";
        args["referer"]              = "https://ui.ptlogin2.qq.com/cgi-bin/login";
        args["jumpUrl"]              = "https://ui.ptlogin2.qq.com/cgi-bin/hello";
        args["cookieHash"]           = "D692D87319F2098C3877C3904B304706";
        args["userAgent"]            = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.132 Safari/537.36";
        args["xForwardedFor"]        = "121.14.96.121";
        args["mouseClickCount"]      = "10";
        args["keyboardClickCount"]   = "34";
        args["result"]               = "2";
        args["reason"]               = "4";

        /* 设备信息 */
        args["macAddress"]           = "00-05-9A-3C-7A-00";
        args["vendorId"]             = "tencent.com";
        args["imei"]                 = "54654654646";
        args["appVersion"]           = "10.0.1";

        /* 其他信息 */
        args["businessId"]           = "1";

        string url = BspAPI.MakeURL("GET", "RegisterProtection", "gz", SECRET_ID, SECRET_KEY, args);
        string result = SendGet(url);
        Console.WriteLine(result);
    }
}

